@@ -5,12 +5,7 @@
 use anyhow::{anyhow, Context, Result};
 use log::{debug, log_enabled, trace, Level};
 use native_tls::{TlsConnector, TlsStream};
-use std::{
-    collections::HashSet,
-    convert::{TryFrom, TryInto},
-    net::TcpStream,
-    thread,
-};
+use std::{collections::HashSet, convert::TryFrom, net::TcpStream, thread};
 
 use crate::{
     config::{Account, Config},
@@ -21,7 +16,7 @@ use crate::{
 type ImapSession = imap::Session<TlsStream<TcpStream>>;
 
 pub trait ImapServiceInterface<'a> {
-    fn notify(&mut self, config: &Config, keepalive: u64) -> Result<()>;
+    fn notify(&mut self, config: &Config, account: &Account, keepalive: u64) -> Result<()>;
     fn watch(&mut self, account: &Account, keepalive: u64) -> Result<()>;
     fn fetch_mboxes(&'a mut self) -> Result<Mboxes>;
     fn fetch_envelopes(&mut self, page_size: &usize, page: &usize) -> Result<Envelopes>;
@@ -31,9 +26,9 @@ pub trait ImapServiceInterface<'a> {
         page_size: &usize,
         page: &usize,
     ) -> Result<Envelopes>;
-    fn find_msg(&mut self, seq: &str) -> Result<Msg>;
+    fn find_msg(&mut self, account: &Account, seq: &str) -> Result<Msg>;
     fn find_raw_msg(&mut self, seq: &str) -> Result<Vec<u8>>;
-    fn append_msg(&mut self, mbox: &Mbox, msg: Msg) -> Result<()>;
+    fn append_msg(&mut self, mbox: &Mbox, account: &Account, msg: Msg) -> Result<()>;
     fn append_raw_msg_with_flags(&mut self, mbox: &Mbox, msg: &[u8], flags: Flags) -> Result<()>;
     fn expunge(&mut self) -> Result<()>;
     fn logout(&mut self) -> Result<()>;
@@ -98,10 +93,10 @@ impl<'a> ImapService<'a> {
         }
     }
 
-    fn search_new_msgs(&mut self) -> Result<Vec<u32>> {
+    fn search_new_msgs(&mut self, account: &Account) -> Result<Vec<u32>> {
         let uids: Vec<u32> = self
             .sess()?
-            .uid_search("NEW")
+            .uid_search(&account.notify_query)
             .context("cannot search new messages")?
             .into_iter()
             .collect();
@@ -197,11 +192,11 @@ impl<'a> ImapServiceInterface<'a> for ImapService<'a> {
     }
 
     /// Find a message by sequence number.
-    fn find_msg(&mut self, seq: &str) -> Result<Msg> {
+    fn find_msg(&mut self, account: &Account, seq: &str) -> Result<Msg> {
         let mbox = self.mbox.to_owned();
         self.sess()?
             .select(&mbox.name)
-            .context(format!(r#"cannot select mailbox "{}""#, self.mbox.name))?;
+            .context(format!("cannot select mailbox {}", self.mbox.name))?;
         let fetches = self
             .sess()?
             .fetch(seq, "(ENVELOPE FLAGS INTERNALDATE BODY[])")
@@ -210,7 +205,7 @@ impl<'a> ImapServiceInterface<'a> for ImapService<'a> {
             .first()
             .ok_or_else(|| anyhow!(r#"cannot find message "{}"#, seq))?;
 
-        Msg::try_from(fetch)
+        Msg::try_from((account, fetch))
     }
 
     fn find_raw_msg(&mut self, seq: &str) -> Result<Vec<u8>> {
@@ -238,8 +233,8 @@ impl<'a> ImapServiceInterface<'a> for ImapService<'a> {
         Ok(())
     }
 
-    fn append_msg(&mut self, mbox: &Mbox, msg: Msg) -> Result<()> {
-        let msg_raw: Vec<u8> = (&msg).try_into()?;
+    fn append_msg(&mut self, mbox: &Mbox, account: &Account, msg: Msg) -> Result<()> {
+        let msg_raw = msg.into_sendable_msg(account)?.formatted();
         self.sess()?
             .append(&mbox.name, &msg_raw)
             .flags(msg.flags.0)
@@ -248,7 +243,7 @@ impl<'a> ImapServiceInterface<'a> for ImapService<'a> {
         Ok(())
     }
 
-    fn notify(&mut self, config: &Config, keepalive: u64) -> Result<()> {
+    fn notify(&mut self, config: &Config, account: &Account, keepalive: u64) -> Result<()> {
         debug!("notify");
 
         let mbox = self.mbox.to_owned();
@@ -260,7 +255,7 @@ impl<'a> ImapServiceInterface<'a> for ImapService<'a> {
 
         debug!("init messages hashset");
         let mut msgs_set: HashSet<u32> = self
-            .search_new_msgs()?
+            .search_new_msgs(account)?
             .iter()
             .cloned()
             .collect::<HashSet<_>>();
@@ -281,7 +276,7 @@ impl<'a> ImapServiceInterface<'a> for ImapService<'a> {
                 .context("cannot start the idle mode")?;
 
             let uids: Vec<u32> = self
-                .search_new_msgs()?
+                .search_new_msgs(account)?
                 .into_iter()
                 .filter(|uid| -> bool { msgs_set.get(uid).is_none() })
                 .collect();
