@@ -4,7 +4,7 @@
 
 use anyhow::Result;
 use clap::{self, App, Arg, ArgMatches, SubCommand};
-use log::{debug, trace};
+use log::{debug, info, trace};
 
 use crate::{
     domain::{
@@ -25,21 +25,22 @@ type RawMsg<'a> = &'a str;
 type Query = String;
 type AttachmentPaths<'a> = Vec<&'a str>;
 type MaxTableWidth = Option<usize>;
+type Encrypt = bool;
 
 /// Message commands.
 pub enum Command<'a> {
     Attachments(Seq<'a>),
     Copy(Seq<'a>, Mbox<'a>),
     Delete(Seq<'a>),
-    Forward(Seq<'a>, AttachmentPaths<'a>),
+    Forward(Seq<'a>, AttachmentPaths<'a>, Encrypt),
     List(MaxTableWidth, Option<PageSize>, Page),
     Move(Seq<'a>, Mbox<'a>),
     Read(Seq<'a>, TextMime<'a>, Raw),
-    Reply(Seq<'a>, All, AttachmentPaths<'a>),
+    Reply(Seq<'a>, All, AttachmentPaths<'a>, Encrypt),
     Save(RawMsg<'a>),
     Search(Query, MaxTableWidth, Option<PageSize>, Page),
     Send(RawMsg<'a>),
-    Write(AttachmentPaths<'a>),
+    Write(AttachmentPaths<'a>, Encrypt),
 
     Flag(Option<flag_arg::Command<'a>>),
     Tpl(Option<tpl_arg::Command<'a>>),
@@ -47,46 +48,50 @@ pub enum Command<'a> {
 
 /// Message command matcher.
 pub fn matches<'a>(m: &'a ArgMatches) -> Result<Option<Command<'a>>> {
+    info!("entering message command matcher");
+
     if let Some(m) = m.subcommand_matches("attachments") {
-        debug!("attachments command matched");
+        info!("attachments command matched");
         let seq = m.value_of("seq").unwrap();
-        trace!("seq: {}", seq);
+        debug!("seq: {}", seq);
         return Ok(Some(Command::Attachments(seq)));
     }
 
     if let Some(m) = m.subcommand_matches("copy") {
-        debug!("copy command matched");
+        info!("copy command matched");
         let seq = m.value_of("seq").unwrap();
-        trace!("seq: {}", seq);
+        debug!("seq: {}", seq);
         let mbox = m.value_of("mbox-target").unwrap();
-        trace!(r#"target mailbox: "{:?}""#, mbox);
+        debug!(r#"target mailbox: "{:?}""#, mbox);
         return Ok(Some(Command::Copy(seq, mbox)));
     }
 
     if let Some(m) = m.subcommand_matches("delete") {
-        debug!("copy command matched");
+        info!("copy command matched");
         let seq = m.value_of("seq").unwrap();
-        trace!("seq: {}", seq);
+        debug!("seq: {}", seq);
         return Ok(Some(Command::Delete(seq)));
     }
 
     if let Some(m) = m.subcommand_matches("forward") {
-        debug!("forward command matched");
+        info!("forward command matched");
         let seq = m.value_of("seq").unwrap();
-        trace!("seq: {}", seq);
+        debug!("seq: {}", seq);
         let paths: Vec<&str> = m.values_of("attachments").unwrap_or_default().collect();
-        trace!("attachments paths: {:?}", paths);
-        return Ok(Some(Command::Forward(seq, paths)));
+        debug!("attachments paths: {:?}", paths);
+        let encrypt = m.is_present("encrypt");
+        debug!("encrypt: {}", encrypt);
+        return Ok(Some(Command::Forward(seq, paths, encrypt)));
     }
 
     if let Some(m) = m.subcommand_matches("list") {
-        debug!("list command matched");
+        info!("list command matched");
         let max_table_width = m
             .value_of("max-table-width")
             .and_then(|width| width.parse::<usize>().ok());
-        trace!(r#"max table width: "{:?}""#, max_table_width);
+        debug!("max table width: {:?}", max_table_width);
         let page_size = m.value_of("page-size").and_then(|s| s.parse().ok());
-        trace!(r#"page size: "{:?}""#, page_size);
+        debug!("page size: {:?}", page_size);
         let page = m
             .value_of("page")
             .unwrap_or("1")
@@ -94,56 +99,59 @@ pub fn matches<'a>(m: &'a ArgMatches) -> Result<Option<Command<'a>>> {
             .ok()
             .map(|page| 1.max(page) - 1)
             .unwrap_or_default();
-        trace!(r#"page: "{:?}""#, page);
+        debug!("page: {}", page);
         return Ok(Some(Command::List(max_table_width, page_size, page)));
     }
 
     if let Some(m) = m.subcommand_matches("move") {
-        debug!("move command matched");
+        info!("move command matched");
         let seq = m.value_of("seq").unwrap();
-        trace!("seq: {}", seq);
+        debug!("seq: {}", seq);
         let mbox = m.value_of("mbox-target").unwrap();
-        trace!(r#"target mailbox: "{:?}""#, mbox);
+        debug!("target mailbox: {:?}", mbox);
         return Ok(Some(Command::Move(seq, mbox)));
     }
 
     if let Some(m) = m.subcommand_matches("read") {
-        debug!("read command matched");
+        info!("read command matched");
         let seq = m.value_of("seq").unwrap();
-        trace!("seq: {}", seq);
+        debug!("seq: {}", seq);
         let mime = m.value_of("mime-type").unwrap();
-        trace!("text mime: {}", mime);
+        debug!("text mime: {}", mime);
         let raw = m.is_present("raw");
-        trace!("raw: {}", raw);
+        debug!("raw: {}", raw);
         return Ok(Some(Command::Read(seq, mime, raw)));
     }
 
     if let Some(m) = m.subcommand_matches("reply") {
-        debug!("reply command matched");
+        info!("reply command matched");
         let seq = m.value_of("seq").unwrap();
-        trace!("seq: {}", seq);
+        debug!("seq: {}", seq);
         let all = m.is_present("reply-all");
-        trace!("reply all: {}", all);
+        debug!("reply all: {}", all);
         let paths: Vec<&str> = m.values_of("attachments").unwrap_or_default().collect();
-        trace!("attachments paths: {:#?}", paths);
-        return Ok(Some(Command::Reply(seq, all, paths)));
+        debug!("attachments paths: {:?}", paths);
+        let encrypt = m.is_present("encrypt");
+        debug!("encrypt: {}", encrypt);
+
+        return Ok(Some(Command::Reply(seq, all, paths, encrypt)));
     }
 
     if let Some(m) = m.subcommand_matches("save") {
-        debug!("save command matched");
+        info!("save command matched");
         let msg = m.value_of("message").unwrap_or_default();
         trace!("message: {}", msg);
         return Ok(Some(Command::Save(msg)));
     }
 
     if let Some(m) = m.subcommand_matches("search") {
-        debug!("search command matched");
+        info!("search command matched");
         let max_table_width = m
             .value_of("max-table-width")
             .and_then(|width| width.parse::<usize>().ok());
-        trace!(r#"max table width: "{:?}""#, max_table_width);
+        debug!("max table width: {:?}", max_table_width);
         let page_size = m.value_of("page-size").and_then(|s| s.parse().ok());
-        trace!(r#"page size: "{:?}""#, page_size);
+        debug!("page size: {:?}", page_size);
         let page = m
             .value_of("page")
             .unwrap()
@@ -151,7 +159,7 @@ pub fn matches<'a>(m: &'a ArgMatches) -> Result<Option<Command<'a>>> {
             .ok()
             .map(|page| 1.max(page) - 1)
             .unwrap_or_default();
-        trace!(r#"page: "{:?}""#, page);
+        debug!("page: {}", page);
         let query = m
             .values_of("query")
             .unwrap_or_default()
@@ -176,7 +184,7 @@ pub fn matches<'a>(m: &'a ArgMatches) -> Result<Option<Command<'a>>> {
             })
             .1
             .join(" ");
-        trace!(r#"query: "{:?}""#, query);
+        debug!("query: {}", query);
         return Ok(Some(Command::Search(
             query,
             max_table_width,
@@ -186,17 +194,19 @@ pub fn matches<'a>(m: &'a ArgMatches) -> Result<Option<Command<'a>>> {
     }
 
     if let Some(m) = m.subcommand_matches("send") {
-        debug!("send command matched");
+        info!("send command matched");
         let msg = m.value_of("message").unwrap_or_default();
         trace!("message: {}", msg);
         return Ok(Some(Command::Send(msg)));
     }
 
     if let Some(m) = m.subcommand_matches("write") {
-        debug!("write command matched");
+        info!("write command matched");
         let attachment_paths: Vec<&str> = m.values_of("attachments").unwrap_or_default().collect();
-        trace!("attachments paths: {:?}", attachment_paths);
-        return Ok(Some(Command::Write(attachment_paths)));
+        debug!("attachments paths: {:?}", attachment_paths);
+        let encrypt = m.is_present("encrypt");
+        debug!("encrypt: {}", encrypt);
+        return Ok(Some(Command::Write(attachment_paths, encrypt)));
     }
 
     if let Some(m) = m.subcommand_matches("template") {
@@ -207,7 +217,7 @@ pub fn matches<'a>(m: &'a ArgMatches) -> Result<Option<Command<'a>>> {
         return Ok(Some(Command::Flag(flag_arg::matches(m)?)));
     }
 
-    debug!("default list command matched");
+    info!("default list command matched");
     Ok(Some(Command::List(None, None, 0)))
 }
 
@@ -265,6 +275,14 @@ pub fn attachment_arg<'a>() -> Arg<'a, 'a> {
         .multiple(true)
 }
 
+/// Message encrypt argument.
+pub fn encrypt_arg<'a>() -> Arg<'a, 'a> {
+    Arg::with_name("encrypt")
+        .help("Encrypts the message")
+        .short("e")
+        .long("encrypt")
+}
+
 /// Message subcommands.
 pub fn subcmds<'a>() -> Vec<App<'a, 'a>> {
     vec![
@@ -297,7 +315,8 @@ pub fn subcmds<'a>() -> Vec<App<'a, 'a>> {
                 ),
             SubCommand::with_name("write")
                 .about("Writes a new message")
-                .arg(attachment_arg()),
+                .arg(attachment_arg())
+                .arg(encrypt_arg()),
             SubCommand::with_name("send")
                 .about("Sends a raw message")
                 .arg(Arg::with_name("message").raw(true).last(true)),
@@ -327,12 +346,14 @@ pub fn subcmds<'a>() -> Vec<App<'a, 'a>> {
                 .about("Answers to a message")
                 .arg(seq_arg())
                 .arg(reply_all_arg())
-                .arg(attachment_arg()),
+                .arg(attachment_arg())
+		.arg(encrypt_arg()),
             SubCommand::with_name("forward")
                 .aliases(&["fwd", "f"])
                 .about("Forwards a message")
                 .arg(seq_arg())
-                .arg(attachment_arg()),
+                .arg(attachment_arg())
+		.arg(encrypt_arg()),
             SubCommand::with_name("copy")
                 .aliases(&["cp", "c"])
                 .about("Copies a message to the targetted mailbox")
